/**************************************************************************
* libXicq - Originally written by tnc (xtrophy@it.dk)                     *
*'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''*
* libXicq.cc                                                              *
* Glues the library together                                              *
**************************************************************************/


#include "libXicq.h"


struct LIB_INFORMATION lib_information;
struct NET_INFORMATION net_information;
struct USER_CONFIGURATION user_config;
struct CALLBACK_FUNCTIONS callback;

// Filedescripters for select()
struct timeval tv;
fd_set fd1, fd2;

// Used to time 'keep alive' packets
time_t keepAliveTime;

/*
**
** Sets up the callback functions, used to send info to the UI.
*/
void setupCallbacks(struct CALLBACK_FUNCTIONS cbf)
{
    callback = cbf;
}

/*
**
** Initializes the library
** Returns -1 if something goes wrong
*/
int icq_init(struct USER_CONFIGURATION *user_cfg)
{
/*Om gvrning av lib infon, det ger inte att ha det som dom har av Maos*/
    lib_information.NAME=LNAME;
    lib_information.VERSION=LVERSION;
    lib_information.AUTHOR=LAUTHOR;
    lib_information.CONTACT=LCONTACT;

    if (DEBUG)
        icq_printInfo();

    user_config = *user_cfg;
    user_config.status = STATUS_OFFLINE;        // Not connected yet

    if (DEBUG)
    {
	printf("UIN  : %ld\n", user_config.uin);
/*	printf("PWD  : %s\n", user_config.password);*/
	printf("NICK : %s\n", user_config.nick);
	printf("FIRST: %s\n", user_config.firstName);
	printf("LAST : %s\n", user_config.lastName);
	printf("EMAIL: %s\n", user_config.email);
	printf("STAT : %ld\n", user_config.defaultStatus);
	printf("LIST : %s\n", user_config.contactList);
	printf("ADDR : %s\n", user_config.serverAddr);
	printf("PORT : %d\n", user_config.serverPort);
    }

    if (net_init(&user_config) == -1)     // Initialize networking
        return -1;

    contact_init(user_config.contactList);

    // Set up FDs for select()
    FD_ZERO(&fd2);
    FD_SET(net_information.sock, &fd2);
    FD_SET(0, &fd2);

/*Nerflyttad hit av Maos kan inte vara ldngst upp (utanfvr funktioner) utan dr
**nu hdr istdllet dnda problemet jag kan se 'r att fvrsta keepaliven kommer fvr
**tidigt*/
    keepAliveTime = time(NULL);

    if (DEBUG)
        misc_log("Icq initialized.");

    return 0;
}


/*
**
** Prints information about this library
*/
void icq_printInfo()
{
    printf("%s %s by %s (%s )\n",lib_information.NAME,lib_information.VERSION,
           lib_information.AUTHOR, lib_information.CONTACT);
}


/*
** This is to be called whenever the client is idle.
** Get packets if any, parse 'em, do the callback etc
** Returns -1 if something is wrong
*/
int  icq_main()
{
    tv.tv_sec = 0;
    tv.tv_usec = 250;
    fd1 = fd2;

    // See if we got data waiting
    select(net_information.sock + 1, &fd1, NULL, NULL, &tv);

    // We got some data, fetch it
    if(FD_ISSET(net_information.sock, &fd1))
    {
        struct ICQ_PACKET packet;

        packet.length   = net_udpRecv(packet.data, BUFSIZE);
	if (packet.length != 0 && packet.length != -1)
	{
	    packet.command  = misc_packet2cmd(&packet);
	    packet.sequence = misc_packet2seq(&packet);

	    // Let's get the packet parsed.
	    parse(&packet);
	}
    }

    // Tell the server we're still here
    if(keepAliveTime+90 < time(NULL)) printf("Keepaliven missad!\n");
    if (keepAliveTime + 85 < time(NULL))
    {
        send_keepAlive();
        keepAliveTime = time(NULL);
    }

    return 0;
}



